import 'package:json_annotation/json_annotation.dart';

import "ILedgerTbl.dart" show ILedgerTbl;
import "IR5Etab.dart" show IR5Etab;
import "IR5Staff.dart" show IR5Staff;
import "IR5Station.dart" show IR5Station;

part 'ICDisplay.g.dart';

@JsonSerializable()
class IR5WampBalanceEvent {
  @JsonKey(defaultValue: '')
  final String PLU;
  @JsonKey(defaultValue: '')
  final String NomPLU;
  @JsonKey(defaultValue: '')
  final String Image;
  @JsonKey(defaultValue: 0)
  final double Quantity;
  @JsonKey(defaultValue: 0)
  final double UnitPrice;
  @JsonKey(defaultValue: 0)
  final double TotalBase;
  @JsonKey(defaultValue: 0)
  final double TotalTTC;

  const IR5WampBalanceEvent({
    required this.PLU,
    required this.NomPLU,
    required this.Image,
    required this.Quantity,
    required this.UnitPrice,
    required this.TotalBase,
    required this.TotalTTC,
  });

  factory IR5WampBalanceEvent.fromJson([Map<String, dynamic>? json]) => _$IR5WampBalanceEventFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$IR5WampBalanceEventToJson(this);
}

@JsonSerializable()
class IR5WampPayEvent {
  @JsonKey(defaultValue: '')
  final String Machine;
  @JsonKey(defaultValue: 0)
  final double ToPaid;
  @JsonKey(defaultValue: 0)
  final double IN;
  @JsonKey(defaultValue: 0)
  final double OUT;

  const IR5WampPayEvent({
    required this.Machine,
    required this.ToPaid,
    required this.IN,
    required this.OUT,
  });

  factory IR5WampPayEvent.fromJson([Map<String, dynamic>? json]) => _$IR5WampPayEventFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$IR5WampPayEventToJson(this);
}

@JsonSerializable(explicitToJson: true)
class IR5WampDataEvent {
  final IR5Etab Etab;
  final dynamic Frame;
  final IR5Station Station;
  final IR5Staff Staff;
  final IR5WampPayEvent Pay;
  final IR5WampBalanceEvent Balance;
  final ILedgerTbl Tbl;

  const IR5WampDataEvent({
    required this.Etab,
    this.Frame,
    required this.Station,
    required this.Staff,
    required this.Pay,
    required this.Balance,
    required this.Tbl,
  });

  factory IR5WampDataEvent.fromJson([Map<String, dynamic>? json]) => _$IR5WampDataEventFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$IR5WampDataEventToJson(this);
}
