import 'package:json_annotation/json_annotation.dart';

import "IR5Etab.dart" show IR5Etab;
import "IR5EtabFDM.dart" show IR5EtabFDM;
import "IR5ModePay.dart" show IR5ModePay;
import "IR5Staff.dart" show IR5Staff;
import "IR5Station.dart" show IR5Station;

part 'IDashboardIntf.g.dart';

// --------------------------------------------------------------------------------
@JsonSerializable(explicitToJson: true)
class IDashboardData {
  final IR5Etab? Etab;
  final IR5EtabFDM? EtabIntf;
  @JsonKey(defaultValue: [])
  final List<IR5Staff> Staffs;
  @JsonKey(defaultValue: [])
  final List<IR5Station> Stations;
  @JsonKey(defaultValue: [])
  final List<IR5ModePay> ModePays;

  const IDashboardData({
    this.Etab,
    this.EtabIntf,
    required this.Staffs,
    required this.Stations,
    required this.ModePays,
  });

  factory IDashboardData.fromJson([Map<String, dynamic>? json]) => _$IDashboardDataFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$IDashboardDataToJson(this);
}

// --------------------------------------------------------------------------------
@JsonSerializable()
class IDashboardQuery {
  /** Array of Data {DateService,Conso,PAY,Tbls,CVts} */
  @JsonKey(defaultValue: [])
  final List<dynamic> Data;
  /** Data Definition {DateService,Conso,PAY,Tbls,CVts} */
  final dynamic MetaData;

  const IDashboardQuery({
    required this.Data,
    this.MetaData,
  });

  factory IDashboardQuery.fromJson([Map<String, dynamic>? json]) => _$IDashboardQueryFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$IDashboardQueryToJson(this);
}

// --------------------------------------------------------------------------------
@JsonSerializable()
class IDashboardDBInfo {
  @JsonKey(defaultValue: '')
  final String dbVer;
  @JsonKey(defaultValue: '')
  final String CMAXNum;
  @JsonKey(defaultValue: '')
  final String EtabName;
  @JsonKey(defaultValue: '')
  final String EOL;
  @JsonKey(defaultValue: 0)
  final int ServiceStatus;
  @JsonKey(defaultValue: '')
  final String ServiceInfo;
  @JsonKey(defaultValue: 0)
  final int ServiceNo;
  @JsonKey(defaultValue: '')
  final String ServiceNom;
  @JsonKey(defaultValue: '')
  final String ServiceDate;
  @JsonKey(defaultValue: '')
  final String PrevServiceDate;
  @JsonKey(defaultValue: 0)
  final int NbTbls;
  @JsonKey(defaultValue: 0)
  final int NbCvts;
  @JsonKey(defaultValue: 0)
  final int NbArchives;
  @JsonKey(defaultValue: 0)
  final int NbHistoDays;
  @JsonKey(defaultValue: '')
  final String OldestHisto;
  @JsonKey(defaultValue: 0)
  final int NewestHisto;

  const IDashboardDBInfo({
    required this.dbVer,
    required this.CMAXNum,
    required this.EtabName,
    required this.EOL,
    required this.ServiceStatus,
    required this.ServiceInfo,
    required this.ServiceNo,
    required this.ServiceNom,
    required this.ServiceDate,
    required this.PrevServiceDate,
    required this.NbTbls,
    required this.NbCvts,
    required this.NbArchives,
    required this.NbHistoDays,
    required this.OldestHisto,
    required this.NewestHisto,
  });

  factory IDashboardDBInfo.fromJson([Map<String, dynamic>? json]) => _$IDashboardDBInfoFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$IDashboardDBInfoToJson(this);
}

// --------------------------------------------------------------------------------
@JsonSerializable()
class IDashboardConsoTot {
  @JsonKey(defaultValue: '')
  final String Name;
  @JsonKey(defaultValue: 0)
  final double Qty;
  @JsonKey(defaultValue: 0)
  final double Base;
  @JsonKey(defaultValue: 0)
  final double Tax;
  @JsonKey(defaultValue: 0)
  final double TTC;

  const IDashboardConsoTot({
    required this.Name,
    required this.Qty,
    required this.Base,
    required this.Tax,
    required this.TTC,
  });

  factory IDashboardConsoTot.fromJson([Map<String, dynamic>? json]) => _$IDashboardConsoTotFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$IDashboardConsoTotToJson(this);
}

// --------------------------------------------------------------------------------
@JsonSerializable()
class IDashboardTaxVentil {
  @JsonKey(defaultValue: 0)
  final double Rate;
  @JsonKey(defaultValue: 0)
  final double Base;
  @JsonKey(defaultValue: 0)
  final double Tax;
  @JsonKey(defaultValue: 0)
  final double TTC;

  const IDashboardTaxVentil({
    required this.Rate,
    required this.Base,
    required this.Tax,
    required this.TTC,
  });

  factory IDashboardTaxVentil.fromJson([Map<String, dynamic>? json]) => _$IDashboardTaxVentilFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$IDashboardTaxVentilToJson(this);
}

// --------------------------------------------------------------------------------
@JsonSerializable()
class IDashboardPayTot {
  @JsonKey(defaultValue: '')
  final String Name;
  @JsonKey(defaultValue: 0)
  final double Qty;
  @JsonKey(defaultValue: 0)
  final double Tot;

  const IDashboardPayTot({
    required this.Name,
    required this.Qty,
    required this.Tot,
  });

  factory IDashboardPayTot.fromJson([Map<String, dynamic>? json]) => _$IDashboardPayTotFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$IDashboardPayTotToJson(this);
}

// --------------------------------------------------------------------------------
@JsonSerializable()
class IDashboardSum {
  @JsonKey(defaultValue: 0)
  final int Tbls;
  @JsonKey(defaultValue: 0)
  final int Cvts;
  @JsonKey(defaultValue: 0)
  final int Suites;
  @JsonKey(defaultValue: 0)
  final double ConsoBase;
  @JsonKey(defaultValue: 0)
  final double ConsoTax;
  @JsonKey(defaultValue: 0)
  final double ConsoTTC;
  @JsonKey(defaultValue: 0)
  final double Pay;

  const IDashboardSum({
    required this.Tbls,
    required this.Cvts,
    required this.Suites,
    required this.ConsoBase,
    required this.ConsoTax,
    required this.ConsoTTC,
    required this.Pay,
  });

  factory IDashboardSum.fromJson([Map<String, dynamic>? json]) => _$IDashboardSumFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$IDashboardSumToJson(this);
}

// --------------------------------------------------------------------------------
@JsonSerializable(explicitToJson: true)
class IDashboardSvcInfo {
  final IDashboardSum Open;
  final IDashboardSum Tot;
  @JsonKey(defaultValue: [])
  final List<IDashboardTaxVentil> Ventil;
  @JsonKey(defaultValue: [])
  final List<IDashboardPayTot> MPs;
  @JsonKey(defaultValue: [])
  final List<IDashboardConsoTot> Consos;

  const IDashboardSvcInfo({
    required this.Open,
    required this.Tot,
    required this.Ventil,
    required this.MPs,
    required this.Consos,
  });

  factory IDashboardSvcInfo.fromJson([Map<String, dynamic>? json]) => _$IDashboardSvcInfoFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$IDashboardSvcInfoToJson(this);
}

// --------------------------------------------------------------------------------
@JsonSerializable(explicitToJson: true)
class IDashboard {
  /** Etab,Stations,Staffs,... Data */
  final IDashboardData Data;
  /** More DB Info... */
  final IDashboardDBInfo DBInfo;
  /** Open Service summary */
  final IDashboardSvcInfo SvcInfo;
  /** Last 30 days histo summary */
  final IDashboardQuery Histos;

  const IDashboard({
    required this.Data,
    required this.DBInfo,
    required this.SvcInfo,
    required this.Histos,
  });

  factory IDashboard.fromJson([Map<String, dynamic>? json]) => _$IDashboardFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$IDashboardToJson(this);
}
