import 'package:json_annotation/json_annotation.dart';

part 'ILedgerIntf.g.dart';

// --------------------------------------------------------------------------------
@JsonSerializable()
class LedgerSeqKey {
  /** Pointage Timestamp as String(yyyy-mm-dd hh:nn:ss) or as firestore.TimeStamp or ... */
  final String? DateHeure;
  final int? NoSeq;

  const LedgerSeqKey({this.DateHeure, this.NoSeq});

  factory LedgerSeqKey.fromJson([Map<String, dynamic>? json]) => _$LedgerSeqKeyFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$LedgerSeqKeyToJson(this);
}

// --------------------------------------------------------------------------------
@JsonSerializable()
class LedgerSummary {
  @JsonKey(defaultValue: 0)
  final int Tbls;
  @JsonKey(defaultValue: 0)
  final int Cvts;
  @JsonKey(defaultValue: 0)
  final double Base;
  @JsonKey(defaultValue: 0)
  final double TTC;
  @JsonKey(defaultValue: 0)
  final double Pay;

  const LedgerSummary({
    required this.Tbls,
    required this.Cvts,
    required this.Base,
    required this.TTC,
    required this.Pay,
  });

  factory LedgerSummary.fromJson([Map<String, dynamic>? json]) => _$LedgerSummaryFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$LedgerSummaryToJson(this);
}
// --------------------------------------------------------------------------------

/** Histo aka Big Book */
@JsonSerializable(explicitToJson: true)
class ILedgerSalesConso {
  /** Histo Action Flag
   *   O : Ouverture
   *   > : ReOuv
   *   L : Lancer
   *   + : Suite
   *   S : Split
   *   s : Transft
   *   z : FullTicket
   *   F : Pro Forma
   *   A : Addition
   *   C : Clôture
   *   D : Article
   *   M : Menu
   *   1 : Au Menu
   *   X : Article Externe
   *   Z : Article Container
   *   e : Hotel Room
   *   m : Mémo
   *   G : Message Simple
   *   g : Message.Linked
   *   K : Message Plus
   *   ! : Message Minus
   *   Q : Message Multiplic.Quantite
   *   U : Message Multiplic.Prix
   *   a : Message Au Menu
   *   R : Discount
   *   ^ : Multiplic.Discount
   *   % : Member Discount
   *   V : RestoPass Discount
   *   b : Voucher Discount
   *   ? : Info
   *   - : Séparation Conso
   *   = : Séparation Pay
   *   f : PxCM
   *   x : Drapeau
   *   N : TimeDrapeau
   *   q : TelNrDrapeau
   *   , : Réservation
   *   t : Info Room
   *   # : Info Member
   *   B : Info FreeVoucher
   *   I : Invoice
   *   n : Delivry/Take-Away Info
   *   H : Paiement
   *   H : Paiement Machine
   *   H : Paiement Jeton
   *   P : Paiement Member Account
   *   Y : Bon KDO
   *   € : Débour
   *   { : Ouv.Service
   *   } : Clot.Svc
   *   | : Clot.Box
   *   ( : Staff In
   *   ) : Staff Out
   *   @ : Info Prestation Staff
   *   [ : Staff Clock IN
   *   ] : Staff Clock Out
   *   & : Info Blocage
   *   . : Barcontrol IN
   *   / : Barcontrol REJECTED
   *   $ : Barcontrol OUT
   *   T : Tiroir
   *   p : Rpt
   *   J : Ticket Spl.
   *   k : Addition Duplicata
   *   i : Log Invoice Msg
   *   j : Log RePrint InvTicket Msg
   */
  @JsonKey(defaultValue: '')
  final String Action;
  @JsonKey(defaultValue: '')
  final String PLU;

  final String? Nom;
  final String? NomProd;
  final String? NomAdd;

  /** Histo RT Flags
   *   Bit 00 : HasFDMInfo
   *   Bit 01 : HappyTimeOff
   *   Bit 02 : HappyTimeOn
   *   Bit 03 : IsHappyPrice
   *   Bit 04 : IsHappyQt
   *   Bit 05 : SplittedOut
   *   Bit 06 : SplittedIn
   *   Bit 07 : IsTVAMixed
   *   Bit 08 : IsTVAMixedHeader
   *   Bit 09 : IsTVAMixedDetail
   *   Bit 10 : Banket
   *   Bit 11 : ExternalPLU
   *   Bit 12 : SubMenu
   *   Bit 13 : Container
   *   Bit 14 : QtMulti
   *   Bit 15 : PxMulti
   *   Bit 16 : IsHappySpTariff
   *   Bit 17 : Memo
   *   Bit 18 : Divers
   *   Bit 19 : NomSaissi
   *   Bit 20 : ModifPrix
   *   Bit 21 : Splitted
   *   Bit 22 : SplitChg
   *   Bit 23 : IsRed
   *   Bit 24 : IsGreen
   *   Bit 25 : SuperStaff
   *   Bit 26 : ModifTarif
   *   Bit 27 : PrixDiscount
   *   Bit 28 : KitchenFlaggged
   *   Bit 29 : PayJeton
   *   Bit 30 : BonValeur
   *   Bit 31 : Remise
   *   Bit 32 : IsInTaxClass
   *   Bit 33 : IsOutTaxClass
   *   Bit 35 : IsOffert
   *   Bit 36 : IsForceAZero
   *   Bit 37 : Negatif Fiscal
   *   Bit 38 : Negatif
   *   Bit 39 : Negatif After Add.
   *   Bit 40 : FOOD
   *   Bit 41 : NonFOOD
   *   Bit 42 : ALCOOL
   *   Bit 43 : AfterAddition
   *   Bit 45 : DoNotPrint
   *   Bit 46 : Printed
   *   Bit 47 : PrintPending
   *   Bit 48 : HideQtOnTicketProd
   *   Bit 49 : Deluxe
   *   Bit 51 : RptModifFlag1
   *   Bit 52 : RptModifFlag2
   *   Bit 53 : RptModifFlag3
   *   Bit 54 : RptModifFlag4
   *   Bit 55 : ErrLineOrg
   *   Bit 56 : ErrLineNew
   *   Bit 57 : ErrLine
   *   Bit 58 : LineCorr
   *   Bit 59 : StockLastOne
   *   Bit 60 : Visible
   *   Bit 61 : Training
   *   Bit 62 : ExternalPLUSaved
   *   Bit 63 : NeedStock
   */
  @JsonKey(defaultValue: 0)
  final int RTFlags;
  @JsonKey(defaultValue: '')
  final String TaxClass;
  @JsonKey(defaultValue: 0)
  final int TaxLvl;
  @JsonKey(defaultValue: 0)
  final double TaxRate;
  @JsonKey(defaultValue: 0)
  final double Qty;
  @JsonKey(defaultValue: 0)
  final double Base;
  @JsonKey(defaultValue: 0)
  final double TTC;
  @JsonKey(defaultValue: [])
  final List<ILedgerSalesConso> Details;
  @JsonKey(defaultValue: 0)
  final double Pay;

  const ILedgerSalesConso({
    required this.Action,
    required this.PLU,
    required this.Nom,
    required this.NomProd,
    required this.NomAdd,
    required this.RTFlags,
    required this.TaxClass,
    required this.TaxLvl,
    required this.TaxRate,
    required this.Qty,
    required this.Base,
    required this.TTC,
    required this.Details,
    required this.Pay,
  });

  factory ILedgerSalesConso.fromJson([Map<String, dynamic>? json]) => _$ILedgerSalesConsoFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$ILedgerSalesConsoToJson(this);
}

// --------------------------------------------------------------------------------
@JsonSerializable()
class ILedgerSalesPay {
  @JsonKey(defaultValue: '')
  final String Action;
  @JsonKey(defaultValue: '')
  final String PLU;
  @JsonKey(defaultValue: '')
  final String Nom;
  @JsonKey(defaultValue: '')
  final String NomProd;
  @JsonKey(defaultValue: '')
  final String NomAdd;
  @JsonKey(defaultValue: 0)
  final double Qty;
  @JsonKey(defaultValue: 0)
  final double Pay;

  const ILedgerSalesPay({
    required this.Action,
    required this.PLU,
    required this.Nom,
    required this.NomProd,
    required this.NomAdd,
    required this.Qty,
    required this.Pay,
  });

  factory ILedgerSalesPay.fromJson([Map<String, dynamic>? json]) => _$ILedgerSalesPayFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$ILedgerSalesPayToJson(this);
}
// --------------------------------------------------------------------------------

/** Histo aka Big Book */
@JsonSerializable(explicitToJson: true)
class ILedgerSales {
  /** SVC Date as String(YYYY-MM-DD) or as number YYYYMMDD */

  /** Service Date */
  final dynamic /* String | num */ SvcDte;
  @JsonKey(defaultValue: 0)
  final int SvcNo;

  /** Unique Identifier Number of a Cloture Service ( aka Z). */
  @JsonKey(defaultValue: 0)
  final int SvcNumZ;

  /** Histo <OUFTI> State of the Histo record
   * O  -> Opened Tables Details
   * U  -> cUrrent Service Details
   * F  -> Clotured Services Details
   * T  -> Record already in one Archive Transfered
   * I  -> just In case
   */
  @JsonKey(defaultValue: '')
  final String OUFTIState;

  /** Seq Sign = SHA1(this.SignBase) | SHA1(Details[0..n-1].SignBase)
   * SignBase = Action[S1]PLU[S50]NomPLU[S50]Round(Base*100)[D14]Round(TTC*100)[D14]Round(TaxRate*100)[D6]
   */
  @JsonKey(defaultValue: '')
  final String Sign;
  @JsonKey(defaultValue: 0)
  final int NumSales;
  @JsonKey(defaultValue: '')
  final String GUID;

  /** Pointage Timestamp as String(yyyy-mm-dd hh:nn:ss) or as firestore.TimeStamp or ... */

  /** Registration Timestamp */
  @JsonKey(defaultValue: '')
  final String DateHeure;

  /** Internal Number identifying a Block details of a Sale/Benefit */
  @JsonKey(defaultValue: 0)
  final int Hre;

  /** Sequence Number identifying a Block details of a Sale/Benefit
   * This Define a BLOCK in the Visu
   */
  @JsonKey(defaultValue: 0)
  final int NoSeq;

  /** Staff Number */
  @JsonKey(defaultValue: 0)
  final int Staff;
  @JsonKey(defaultValue: '')
  final String StaffName;

  /** Histo Action Flag
   *   O : Ouverture
   *   > : ReOuv
   *   L : Lancer
   *   + : Suite
   *   S : Split
   *   s : Transft
   *   z : FullTicket
   *   F : Pro Forma
   *   A : Addition
   *   C : Clôture
   *   D : Article
   *   M : Menu
   *   1 : Au Menu
   *   X : Article Externe
   *   Z : Article Container
   *   e : Hotel Room
   *   m : Mémo
   *   G : Message Simple
   *   g : Message.Linked
   *   K : Message Plus
   *   ! : Message Minus
   *   Q : Message Multiplic.Quantite
   *   U : Message Multiplic.Prix
   *   a : Message Au Menu
   *   R : Discount
   *   ^ : Multiplic.Discount
   *   % : Member Discount
   *   V : RestoPass Discount
   *   b : Voucher Discount
   *   ? : Info
   *   - : Séparation Conso
   *   = : Séparation Pay
   *   f : PxCM
   *   x : Drapeau
   *   N : TimeDrapeau
   *   q : TelNrDrapeau
   *   , : Réservation
   *   t : Info Room
   *   # : Info Member
   *   B : Info FreeVoucher
   *   I : Invoice
   *   n : Delivry/Take-Away Info
   *   H : Paiement
   *   H : Paiement Machine
   *   H : Paiement Jeton
   *   P : Paiement Member Account
   *   Y : Bon KDO
   *   € : Débour
   *   { : Ouv.Service
   *   } : Clot.Svc
   *   | : Clot.Box
   *   ( : Staff In
   *   ) : Staff Out
   *   @ : Info Prestation Staff
   *   [ : Staff Clock IN
   *   ] : Staff Clock Out
   *   & : Info Blocage
   *   . : Barcontrol IN
   *   / : Barcontrol REJECTED
   *   $ : Barcontrol OUT
   *   T : Tiroir
   *   p : Rpt
   *   J : Ticket Spl.
   *   k : Addition Duplicata
   *   i : Log Invoice Msg
   *   j : Log RePrint InvTicket Msg
   */
  @JsonKey(defaultValue: '')
  final String Action;
  @JsonKey(defaultValue: '')
  final String Actions;
  @JsonKey(defaultValue: '')
  final String Tbl;
  @JsonKey(defaultValue: 0)
  final double Base;
  @JsonKey(defaultValue: 0)
  final double TTC;
  @JsonKey(defaultValue: 0)
  final double Pay;
  @JsonKey(defaultValue: [])
  final List<ILedgerSalesConso> Details;

  const ILedgerSales({
    required this.DateHeure,
    this.SvcDte,
    required this.SvcNo,
    required this.SvcNumZ,
    required this.OUFTIState,
    required this.Sign,
    required this.NumSales,
    required this.GUID,
    required this.Hre,
    required this.NoSeq,
    required this.Staff,
    required this.StaffName,
    required this.Action,
    required this.Actions,
    required this.Tbl,
    required this.Base,
    required this.TTC,
    required this.Pay,
    required this.Details,
  });

  factory ILedgerSales.fromJson([Map<String, dynamic>? json]) => _$ILedgerSalesFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$ILedgerSalesToJson(this);
}
// --------------------------------------------------------------------------------

/** Histo aka Big Book */
@JsonSerializable()
class ILedgerLogInfo {
  /** SVC Date as String(YYYY-MM-DD) or as number YYYYMMDD */

  /** Service Date */
  final dynamic /* String | num */ SvcDte;
  @JsonKey(defaultValue: 0)
  final int SvcNo;

  /** Unique Identifier Number of a Cloture Service ( aka Z). */
  @JsonKey(defaultValue: 0)
  final int SvcNumZ;

  /** Histo <OUFTI> State of the Histo record
   * O  -> Opened Tables Details
   * U  -> cUrrent Service Details
   * F  -> Clotured Services Details
   * T  -> Record already in one Archive Transfered
   * I  -> just In case
   */
  @JsonKey(defaultValue: '')
  final String OUFTIState;

  /** Seq Sign = SHA1(this.SignBase)
   * SignBase = Action[S1]EmptyStr[S50]EmptyStr[S50]0[D14]0[D14]0[D6]
   */
  @JsonKey(defaultValue: '')
  final String Sign;

  /** Pointage Timestamp as String(yyyy-mm-dd hh:nn:ss) or as firestore.TimeStamp or ... */

  /** Registration Timestamp */
  @JsonKey(defaultValue: '')
  final String DateHeure;

  /** Internal Number identifying a Block details of a Sale/Benefit */
  @JsonKey(defaultValue: 0)
  final int Hre;

  /** Sequence Number identifying a Block details of a Sale/Benefit
   * This Define a BLOCK in the Visu
   */
  @JsonKey(defaultValue: 0)
  final int NoSeq;

  /** Staff Number */
  @JsonKey(defaultValue: 0)
  final int Staff;
  @JsonKey(defaultValue: '')
  final String StaffName;

  /** Histo Action Flag
   *   O : Ouverture
   *   > : ReOuv
   *   L : Lancer
   *   + : Suite
   *   S : Split
   *   s : Transft
   *   z : FullTicket
   *   F : Pro Forma
   *   A : Addition
   *   C : Clôture
   *   D : Article
   *   M : Menu
   *   1 : Au Menu
   *   X : Article Externe
   *   Z : Article Container
   *   e : Hotel Room
   *   m : Mémo
   *   G : Message Simple
   *   g : Message.Linked
   *   K : Message Plus
   *   ! : Message Minus
   *   Q : Message Multiplic.Quantite
   *   U : Message Multiplic.Prix
   *   a : Message Au Menu
   *   R : Discount
   *   ^ : Multiplic.Discount
   *   % : Member Discount
   *   V : RestoPass Discount
   *   b : Voucher Discount
   *   ? : Info
   *   - : Séparation Conso
   *   = : Séparation Pay
   *   f : PxCM
   *   x : Drapeau
   *   N : TimeDrapeau
   *   q : TelNrDrapeau
   *   , : Réservation
   *   t : Info Room
   *   # : Info Member
   *   B : Info FreeVoucher
   *   I : Invoice
   *   n : Delivry/Take-Away Info
   *   H : Paiement
   *   H : Paiement Machine
   *   H : Paiement Jeton
   *   P : Paiement Member Account
   *   Y : Bon KDO
   *   € : Débour
   *   { : Ouv.Service
   *   } : Clot.Svc
   *   | : Clot.Box
   *   ( : Staff In
   *   ) : Staff Out
   *   @ : Info Prestation Staff
   *   [ : Staff Clock IN
   *   ] : Staff Clock Out
   *   & : Info Blocage
   *   . : Barcontrol IN
   *   / : Barcontrol REJECTED
   *   $ : Barcontrol OUT
   *   T : Tiroir
   *   p : Rpt
   *   J : Ticket Spl.
   *   k : Addition Duplicata
   *   i : Log Invoice Msg
   *   j : Log RePrint InvTicket Msg
   */
  @JsonKey(defaultValue: '')
  final String Action;
  @JsonKey(defaultValue: '')
  final String PLU;
  @JsonKey(defaultValue: '')
  final String Nom;
  @JsonKey(defaultValue: '')
  final String NomProd;
  @JsonKey(defaultValue: '')
  final String NomAdd;

  const ILedgerLogInfo({
    this.SvcDte,
    required this.SvcNo,
    required this.SvcNumZ,
    required this.OUFTIState,
    required this.Sign,
    required this.DateHeure,
    required this.Hre,
    required this.NoSeq,
    required this.Staff,
    required this.StaffName,
    required this.Action,
    required this.PLU,
    required this.Nom,
    required this.NomProd,
    required this.NomAdd,
  });

  factory ILedgerLogInfo.fromJson([Map<String, dynamic>? json]) => _$ILedgerLogInfoFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$ILedgerLogInfoToJson(this);
}
