import 'package:json_annotation/json_annotation.dart';

import "ILedgerIntf.dart" show ILedgerSales, LedgerSeqKey, LedgerSummary;
import "IR5Etab.dart" show IR5Etab;
import "IR5EtabFDM.dart" show IR5EtabFDM;

part 'ILedgerOSvc.g.dart';

// --------------------------------------------------------------------------------
@JsonSerializable(explicitToJson: true)
class OSvcLedgerParams {
  @JsonKey(defaultValue: 'OSvcLedger')
  final String Mode;
  final LedgerSeqKey LastSeq;

  const OSvcLedgerParams({required this.Mode, required this.LastSeq});

  factory OSvcLedgerParams.fromJson([Map<String, dynamic>? json]) => _$OSvcLedgerParamsFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$OSvcLedgerParamsToJson(this);
}

// --------------------------------------------------------------------------------
@JsonSerializable(explicitToJson: true)
class IOLedger {
  final IR5Etab Etab;
  final IR5EtabFDM EtabFDM;
  final OSvcLedgerParams Params;

  /** SVC Date as String(YYYY-MM-DD) or as number YYYYMMDD */
  final dynamic /* String | num */ SvcDte;
  @JsonKey(defaultValue: 0)
  final int SvcNo;
  @JsonKey(defaultValue: '')
  final String SvcName;
  final dynamic /* Number | SrvcState */ SvcStatus;

  /** Current Open Service Totals Summary */
  final LedgerSeqKey? SvcFirstSeq;
  final LedgerSeqKey? SvcLastSeq;
  final LedgerSummary? SvcTblsOpen;
  final LedgerSummary? SvcTblsClosed;
  final LedgerSummary? SvcTot;

  /** Returned Seqs (according to Params) Totals Summary */
  final LedgerSeqKey? SeqsFirstSeq;
  final LedgerSeqKey? SeqsLastSeq;
  final LedgerSummary? SeqsTot;
  @JsonKey(defaultValue: [])
  final List<ILedgerSales> Seqs;

  const IOLedger({
    required this.Etab,
    required this.EtabFDM,
    required this.Params,
    this.SvcDte,
    required this.SvcNo,
    required this.SvcName,
    this.SvcStatus,
    required this.SvcFirstSeq,
    required this.SvcLastSeq,
    required this.SvcTblsOpen,
    required this.SvcTblsClosed,
    required this.SvcTot,
    required this.SeqsFirstSeq,
    required this.SeqsLastSeq,
    required this.SeqsTot,
    required this.Seqs,
  });

  factory IOLedger.update(Map<String, dynamic> map, [Map<String, dynamic>? source]) {
    source = source ?? {};
    map["Seqs"] = [...source["Seqs"] ?? [], ...map["Seqs"] ?? []];
    return IOLedger.fromJson({...source, ...map});
  }
  factory IOLedger.fromJson([Map<String, dynamic>? json]) => _$IOLedgerFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$IOLedgerToJson(this);
}
