import 'package:json_annotation/json_annotation.dart';

import "ILedgerIntf.dart" show ILedgerSales, LedgerSummary;
import "IR5Etab.dart" show IR5Etab;
import "IR5EtabFDM.dart" show IR5EtabFDM;

part 'ILedgerSvcs.g.dart';

// --------------------------------------------------------------------------------
@JsonSerializable()
class SvcsLedgerParams {
  @JsonKey(defaultValue: 'SvcsLedger')
  final String Mode;
  @JsonKey(defaultValue: '')
  final String D1;
  @JsonKey(defaultValue: '')
  final String D2;

  const SvcsLedgerParams({
    required this.Mode,
    required this.D1,
    required this.D2,
  });

  factory SvcsLedgerParams.fromJson([Map<String, dynamic>? json]) => _$SvcsLedgerParamsFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$SvcsLedgerParamsToJson(this);
}

// --------------------------------------------------------------------------------
@JsonSerializable(explicitToJson: true)
class ISvcLedger {
  /** SVC Date as String(YYYY-MM-DD) or as number YYYYMMDD */
  final dynamic /* String | num */ SvcDte;
  @JsonKey(defaultValue: 0)
  final int SvcNo;
  @JsonKey(defaultValue: 0)
  final int SvcNumZ;
  @JsonKey(defaultValue: '')
  final String OUFTIState;

  /** Seqs Sign = SHA1(Seqs[0..n-1].Sign) */
  @JsonKey(defaultValue: '')
  final String Sign;
  final LedgerSummary SvcTot;
  @JsonKey(defaultValue: [])
  final List<ILedgerSales> Seqs;

  const ISvcLedger({
    this.SvcDte,
    required this.SvcNo,
    required this.SvcNumZ,
    required this.OUFTIState,
    required this.Sign,
    required this.SvcTot,
    required this.Seqs,
  });

  factory ISvcLedger.update(Map<String, dynamic> map, [Map<String, dynamic>? source]) {
    source = source ?? {};
    map["Seqs"] = [...source["Seqs"] ?? [], ...map["Seqs"] ?? []];
    return ISvcLedger.fromJson({...source, ...map});
  }
  factory ISvcLedger.fromJson([Map<String, dynamic>? json]) => _$ISvcLedgerFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$ISvcLedgerToJson(this);
}

// --------------------------------------------------------------------------------
@JsonSerializable(explicitToJson: true)
class ISvcsLedger {
  final IR5Etab Etab;
  final IR5EtabFDM EtabFDM;
  final SvcsLedgerParams Params;
  final LedgerSummary SvcsTot;
  @JsonKey(defaultValue: [])
  final List<ISvcLedger> Svcs;

  const ISvcsLedger({
    required this.Etab,
    required this.EtabFDM,
    required this.Params,
    required this.SvcsTot,
    required this.Svcs,
  });
  factory ISvcsLedger.update(Map<String, dynamic> map, [Map<String, dynamic>? source]) {
    source = source ?? {};
    map["Svcs"] = [...source["Svcs"] ?? [], ...map["Svcs"] ?? []];
    return ISvcsLedger.fromJson({...source, ...map});
  }
  factory ISvcsLedger.fromJson([Map<String, dynamic>? json]) => _$ISvcsLedgerFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$ISvcsLedgerToJson(this);
}
