import 'package:json_annotation/json_annotation.dart';

import "ILedgerIntf.dart" show ILedgerSales, LedgerSeqKey, LedgerSummary;
import "IR5Etab.dart" show IR5Etab;
import "IR5EtabFDM.dart" show IR5EtabFDM;

part 'ILedgerTLSvc.g.dart';

// --------------------------------------------------------------------------------
@JsonSerializable(explicitToJson: true)
class TLSvcLedgerParams {
  @JsonKey(defaultValue: 'TLSvcLedger')
  final String Mode; // = LedgerMode.TLSvcLedger;
  final LedgerSeqKey LastSeq;

  const TLSvcLedgerParams({
    required this.Mode,
    required this.LastSeq,
  });

  factory TLSvcLedgerParams.fromJson([Map<String, dynamic>? json]) => _$TLSvcLedgerParamsFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$TLSvcLedgerParamsToJson(this);
}

// --------------------------------------------------------------------------------
@JsonSerializable(explicitToJson: true)
class ITLLedger {
  final IR5Etab Etab;
  final IR5EtabFDM EtabFDM;
  final TLSvcLedgerParams Params;

  /** Returned Seqs (according to Params) Totals Summary */
  final LedgerSeqKey SeqsFirstSeq;
  final LedgerSeqKey SeqsLastSeq;
  final LedgerSummary SeqsTot;
  @JsonKey(defaultValue: [])
  final List<ILedgerSales> Seqs;

  const ITLLedger({
    required this.Etab,
    required this.EtabFDM,
    required this.Params,
    required this.SeqsFirstSeq,
    required this.SeqsLastSeq,
    required this.SeqsTot,
    required this.Seqs,
  });

  factory ITLLedger.update(Map<String, dynamic> map, [Map<String, dynamic>? source]) {
    source = source ?? {};
    map["Seqs"] = [...source["Seqs"] ?? [], ...map["Seqs"] ?? []];
    return ITLLedger.fromJson({...source, ...map});
  }
  factory ITLLedger.fromJson([Map<String, dynamic>? json]) => _$ITLLedgerFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$ITLLedgerToJson(this);
}
